# Purpose: Integroh class
# Author: Marçal Junior
# Explanation: This class is responsible for the Integroh API requests and responses handling integroh connectors to integrate with services like keycloak and dropbox.
import json, requests
from .exceptions import ValidationError, exception_handler, ConError, POSSIBLE_ERRORS_MESSAGES
from requests.exceptions import ConnectionError

DOC_URL = 'https://integroh.com/docs/integroh-connector-python'
REQUEST_METHODS = ['GET', 'POST', 'PUT', 'DELETE']
SERVICES = ['keycloak', 'dropbox']
NAME = 'integroh-sys'


class IntegrohConnector:
    def __init__(self):
        self.base_url = ''
        self.api_key = ''
        self.authorization = 'Bearer ' + self.api_key 
        self.headers = {
            'Content-Type': 'application/json',
            'Authorization': self.authorization
        }

    def validate_service(self, service):
        if service not in SERVICES:
            raise ValidationError(f'{POSSIBLE_ERRORS_MESSAGES["service_error"]} {DOC_URL}')
        return service
    
    def validate_request_type(self, request_type):
        if request_type not in REQUEST_METHODS:
            raise ValidationError(f'{POSSIBLE_ERRORS_MESSAGES["request_type_error"]} {DOC_URL}')
        return request_type

    def load_initfile(self):
        try:
            with open('integroh.ini', 'r') as f:
                file_data = f.read()
                print('file_data',file_data)
                data = json.loads(file_data)
                print(data)
                self.base_url = data['base_url']
                self.api_key = data['api_key']
                self.headers = {
                    'Content-Type': 'application/json',
                    'Authorization': 'Bearer ' + self.api_key
                }
        except Exception as e:
            raise ValidationError(f"""
            {POSSIBLE_ERRORS_MESSAGES["configuration_file"]} {DOC_URL}""")
    
    @exception_handler
    def request_service(self, request_type, request_url, request_data):
        try:
            url = self.base_url + request_url
            self.validate_request_type(request_type)
            connection = requests.request(request_type, url, headers=self.headers, data=request_data)
            print(url, connection)
            return connection
        except ConnectionError as e:
            ConError(f'{POSSIBLE_ERRORS_MESSAGES["con_error"]} {DOC_URL}')
    
    @exception_handler    
    def validate_service_data(self, service, service_data):
        """Validate service data: check if is not None and if is a dict"""
        if service_data is None or not isinstance(service_data, dict):
            raise ValidationError(f'{POSSIBLE_ERRORS_MESSAGES["service_data_error"]} {DOC_URL}')
        if service == 'keycloak':
            if 'username' not in service_data:
                raise ValidationError(f'{POSSIBLE_ERRORS_MESSAGES["service_data_error"]} {DOC_URL}')
            if 'password' not in service_data:
                raise ValidationError(f'{POSSIBLE_ERRORS_MESSAGES["service_data_error"]} {DOC_URL}')
        if service == 'dropbox':
            if 'access_token' not in service_data:
                raise ValidationError(f'{POSSIBLE_ERRORS_MESSAGES["service_data_error"]} {DOC_URL}')
            if 'path' not in service_data:
                raise ValidationError(f'{POSSIBLE_ERRORS_MESSAGES["service_data_error"]} {DOC_URL}')
            if 'filename' not in service_data:
                raise ValidationError(f'{POSSIBLE_ERRORS_MESSAGES["service_data_error"]} {DOC_URL}')
            if 'file' not in service_data:
                raise ValidationError(f'{POSSIBLE_ERRORS_MESSAGES["service_data_error"]} {DOC_URL}')
        return service_data

    def connect(self, service, request_type, service_data):
        self.validate_service(service)
        self.validate_service_data(service, service_data)
        self.load_initfile()
        return self.request_service(request_type, service, json.dumps(service_data) if service_data else None)
        
