import unittest
from unittest.mock import patch
from src.intlib.integroh_connector import IntegrohConnector
from src.intlib.exceptions import *

class TestIntegrohConnector(unittest.TestCase):
    def setUp(self):
        self.connector = IntegrohConnector()

    def test_validate_service_valid(self):
        service = 'keycloak'
        result = self.connector.validate_service(service)
        self.assertEqual(result, service)

    def test_validate_service_invalid(self):
        service = 'invalid_service'
        with self.assertRaises(SystemExit):
            self.connector.validate_service(service)

    def test_validate_request_type_valid(self):
        request_type = 'GET'
        result = self.connector.validate_request_type(request_type)
        self.assertEqual(result, request_type)

    def test_validate_request_type_invalid(self):
        request_type = 'INVALID'
        with self.assertRaises(SystemExit):
            self.connector.validate_request_type(request_type)

    @patch('builtins.open')
    def test_load_initfile_success(self, mock_open):
        mock_file_data = '{"base_url": "https://example.com", "api_key": "12345"}'
        mock_open.return_value.__enter__.return_value.read.return_value = mock_file_data
        self.connector.load_initfile()
        self.assertEqual(self.connector.base_url, 'https://example.com')
        self.assertEqual(self.connector.api_key, '12345')

    @patch('builtins.open')
    def test_load_initfile_exception(self, mock_open):
        mock_open.side_effect = ValidationError
        with self.assertRaises(SystemExit):
            self.connector.load_initfile()

    @patch('requests.request')
    def test_request_service_success(self, mock_request):
        mock_request.return_value.status_code = 200
        result = self.connector.request_service('GET', '/api/service', {})
        self.assertEqual(result.status_code, 200)

    @patch('requests.request')
    def test_request_service_connection_error(self, mock_request):
        mock_request.side_effect = ConError
        with self.assertRaises(SystemExit):
            self.connector.request_service('GET', '/api/service', {})

    def test_validate_service_data_valid(self):
        service = 'keycloak'
        service_data = {'username': 'user', 'password': 'pass'}
        result = self.connector.validate_service_data(service, service_data)
        print("Result: ", result)
        self.assertEqual(result, service_data)

    def test_validate_service_data_invalid(self):
        service = 'keycloak'
        service_data = {'username': 'user'}
        with self.assertRaises(SystemExit):
            self.connector.validate_service_data(service, service_data)

    def test_connect_success(self):
        service = 'keycloak'
        request_type = 'GET'
        service_data = {'username': 'user', 'password': 'pass'}
        with patch.object(IntegrohConnector, 'validate_service') as mock_validate_service, \
             patch.object(IntegrohConnector, 'validate_service_data') as mock_validate_service_data, \
             patch.object(IntegrohConnector, 'load_initfile') as mock_load_initfile, \
             patch.object(IntegrohConnector, 'request_service') as mock_request_service:
            mock_validate_service.return_value = service
            mock_validate_service_data.return_value = service_data
            mock_request_service.return_value.status_code = 200
            result = self.connector.connect(service, request_type, service_data)
            self.assertEqual(result.status_code, 200)

if __name__ == '__main__':
    unittest.main()